package com.example.demo;

import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.server.WebSocketServer;

import java.io.UnsupportedEncodingException;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;

public class ServerDemo extends WebSocketServer {


    public ServerDemo() throws UnknownHostException {
    }

    public ServerDemo(int port) throws UnknownHostException {
        super(new InetSocketAddress(port));
        System.out.println("websocket Server start at port:"+port);
    }


    /**
     * 触发连接事件
     */
    @Override
    public void onOpen(WebSocket conn, ClientHandshake clientHandshake) {
        System.out.println("new connection ===" + conn.getRemoteSocketAddress().getAddress().getHostAddress());
    }

    /**
     *
     * 连接断开时触发关闭事件
     */
    @Override
    public void onClose(WebSocket conn, int code, String reason, boolean remote) {
        System.out.println("onClose :" + reason);
    }

    /**
     * 客户端发送消息到服务器时触发事件（心跳）
     */
    @Override
    public void onMessage(WebSocket conn, String message) {
        System.out.println("you have a new message: "+ message);
    }

    /**
     * 客户端发送消息到服务器时触发事件（报警）
     */
    @Override
    public void onMessage(WebSocket conn, ByteBuffer message) {
        System.out.println("you have a new alarm message: "+ message.limit());
        byte[] alert = new byte[message.limit()];
        message.get(alert);

        //header type
        String str = null;
        try {
            str = new String(alert, 0, 8, "utf-8");
        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        //json len
        int json_len = 0;
        for(int i = 0; i < 4; i++) {
            int shift= i * 8;
            json_len +=(alert[i+8] & 0xFF) << shift;
        }

        //seq 
        int seq = 0;
        for(int i = 0; i < 4; i++) {
            int shift= i * 8;
            seq +=(alert[i+12] & 0xFF) << shift;
        }

        System.out.println("type:"+ str + " json length:" + json_len + " seq:" + seq);

        //json 
        String json = null;
        try {
            json = new String(alert, 16, json_len, "utf-8");
        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        System.out.println("json:"+ json);
    }

    /**
     * 触发异常事件
     */
    @Override
    public void onError(WebSocket conn, Exception e) {
        //e.printStackTrace();
        if( conn != null ) {
            //some errors like port binding failed may not be assignable to a specific websocket
        }
    }

    @Override
    public void onStart() {
        System.out.println("Start OK");
    }

    /**
     * 启动服务端
     * @param args
     * @throws UnknownHostException
     */
    public static void main(String[] args) throws UnknownHostException {
        new ServerDemo(8887).start();
    }
}